/* $Id: vga.c,v 1.6 1999/06/21 23:44:57 marcus Exp $
******************************************************************************

   KGI - IBM VGA framebuffer support

   Copyright (C) 1997	Steffen Seeger	[seeger@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <kgi/maintainers.h>
#define	MAINTAINER	Jon_Taylor
#define DRIVER_NAME	"IBM VGA accel driver"
#define	DRIVER_REV	"$Revision: 1.6 $"

/* KGI includes */
#include <kgi/module.h>
#include <kgi/kgi_commands.h>

#include "suggest.inc"
#include "mmio.inc"
/* KGI module interface */

int kgim_accel_init(struct kgi_card *card, struct kgi_display *dpy)
{
	/* FIXME: We should really do something here! */
	NOTICE1("VGA acceleration driver loaded");
	return EOK;
}

void kgim_accel_done(struct kgi_display *dpy)
{
	NOTICE1("VGA acceleration driver removed");
}

int kgim_accel_check_mode(struct kgi_display *dpy, struct kgi_mode *mode,
			  enum kgi_tm_cmd *cmd)
{
	/* !!! Should we check for accel-friendly dimensions here ??? */
	switch (*cmd) 
	{
	case CMD_PROPOSE:
		NOTICE1("Setting up mmio");
		return graphics_mmio_setup(dpy, mode);
	default:
		break;
	}
	
	return EOK;
}

void kgim_accel_set_mode(struct kgi_display *dpy, struct kgi_mode *mode)
{
}

int kgim_accel_command(struct kgi_graphic *gr, kgi_uint cmd, kgi_uint arg)
{ 
	kgi_suggest *sugg;
	const char *name;

	switch (cmd) 
	{
		case ACCEL_GETSUGGEST:
		sugg = (kgi_suggest *)(gr->io_buf);
		gr->io_res = gr->io_buf;
		
		if (sugg->id < 0 || sugg->id > 1)
			return -E(ACCEL, BOGUS);
		if (sugg->id == 1) { /* Generic RAMDAC support */
			strcpy(sugg->name, "generic-ramdac");
			sugg->args[0] = 0;
			sugg->id = 0;
			return 0;
		}
		switch (gr->dev.mode.graphtype) {
			case KGIGT_TEXT16:
			name = sugg_default[SG_TEXT16];
			break;
			case KGIGT_1BIT:
			name = sugg_default[SG_1BIT];
			break;
			case KGIGT_4BIT:
			name = sugg_default[SG_4BIT];
			break;
			case KGIGT_8BIT:
			if (gr->dev.mode.virt.x == 320 &&
			    gr->dev.mode.virt.y == 200)
				name = sugg_default[SG_8BIT];
			else
				name = sugg_default[SG_MODEX];
			break;
			default:
			return -E(ACCEL, BOGUS);
		}
		strcpy(sugg->name, name);
		sugg->args[0] = 0;
		sugg->id = 1;	/* Next is RAMDAC */
		return 0;
		default:
		break;
	}
			
	return -E(ACCEL, NOSUP_ALWAYS_MMAP);
}

